/* eslint-disable @typescript-eslint/no-explicit-any */
import {Request, Response} from "express";
import * as eventService from "../../services/event/eventService";
import {EventRegistrationData} from "../../interfaces/eventInterface";
import {logger} from "../../logger";

export const createManyEvents = async (req: Request, res: Response) => {
  const eventsData: { date: string; hours: string[] } = req.body;
  try {
    const eventId = await eventService.createManyEvents(
      eventsData,
      req.headers.authorization
    );
    logger.info(`Event registered successfully. EventID: ${eventId}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json({message: "Event registered successfully", eventId});
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        eventsData
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({error: error.message});
  }
};

export const linkUserToAEvent = async (req: Request, res: Response) => {
  try {
    const eventId = req.params.id;

    const result = await eventService.linkUserToAEvent(
      eventId,
      req.headers.authorization
    );

    logger.info(`User linked successfully. Result: ${JSON.stringify(result)}`);

    res.status(200).json({message: "User linked successfully", result});
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({error: error.message});
  }
};

export const updateEvent = async (req: Request, res: Response) => {
  try {
    const nEventID = parseInt(req.params.id);
    const eventData: Partial<EventRegistrationData> = req.body;

    const result = await eventService.updateEvent(nEventID, eventData);

    logger.info(`Event edited successfully. Result: ${JSON.stringify(result)}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json({message: "Event updated successfully", result});
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({error: error.message});
  }
};

export const deleteEventById = async (req: Request, res: Response) => {
  try {
    const eventId = parseInt(req.params.id);
    const result = await eventService.deleteEvent(eventId);

    logger.info(
      `Event removed successfully. Result: ${JSON.stringify(result)}`
    );
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json({message: "Event removed successfully", result});
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );

    res.status(400).json({error: error.message});
  }
};

export const get = async (req: Request, res: Response) => {
  try {
    const {authorId, startTime, userType, date, patientId} = req.query as Record<string, any>;

    const result = await eventService.get({
      authorId,
      startTime,
      userType,
      date,
      patientId
    });

    logger.info(
      `Listed all events successfully. Result: ${JSON.stringify(result)}`
    );
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json(result);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({error: error.message});
  }
};