import { Request, Response } from "express";
import * as userService from "../../services/user/userService";
import { UserRegistrationData } from "../../interfaces/userInterface";
import { errorMessages } from "../../../messages/errorMessages";
import { logger } from "../../logger";
import { AppError } from "../../interfaces/appError";
import {User, UserType} from "@prisma/client";

export const registerUser = async (req: Request, res: Response) => {
  try {
    const userData: User & { confirmPassword: string } = req.body;

    const maskedUserData = { ...userData };
    maskedUserData.password = '********';
    maskedUserData.confirmPassword = '********';

    const id = await userService.registerUser(userData);

    logger.info(`User registration request: ${JSON.stringify(maskedUserData)}`);
    logger.info(`User registered successfully. id: ${id}`);
    res.json({ message: "User registered successfully", id });
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(req.body)}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const editUser = async (req: Request, res: Response) => {
  try {
    const userData: UserRegistrationData = req.body;
    const result = await userService.editUser(userData);

    logger.info(`User edited successfully. Result: ${JSON.stringify(result)}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json({ message: "User edited successfully", result });
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const removeUser = async (req: Request, res: Response) => {
  try {
    const userId = parseInt(req.params.id);

    const result = await userService.removeUser(userId);

    logger.info(`User removed successfully. Result: ${JSON.stringify(result)}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json({ message: "User removed successfully", result });
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const listAllUsers = async (req: Request, res: Response) => {
  try {
    const queryParams = req.query;
    const result = await userService.listAllUsers(queryParams);

    logger.info(
      `Listed all users successfully. Result: ${JSON.stringify(result)}`
    );
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json(result);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const listAllProfessional = async (req: Request, res: Response) => {
  try {
    const result = await userService.listAllProfessional();

    logger.info(`Listed all professionals. Result: ${JSON.stringify(result)}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json(result);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({ error: error.message });
  }
};

export const getByUserType = async (req: Request, res: Response) => {

  try {
    const userType:UserType = <"INTEGRATIVE_THERAPIST" | "PSYCHOTHERAPIST" | "ADMIN" | "USER">req.params.userType;

    const result = await userService.listProfessionalByType(userType);

    logger.info(
      `Listed professional by ID successfully. Result: ${JSON.stringify(result)}`
    );
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);
    res.json(result);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(
        error.message,
        error.statusCode
      ),
    });
  }
};

export const listUserByID = async (req: Request, res: Response) => {
  try {
    const userId = parseInt(req.params.id);

    const result = await userService.listUserByID(userId);

    logger.info(
      `Listed user by ID successfully. Result: ${JSON.stringify(result)}`
    );
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json(result);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(400).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const sendRecoveryEmail = async (req: Request, res: Response) => {
  const userBody = { ...req.body };
  userBody.strPassword = "********";

  const { email } = req.body;
  if (!email) {
    return res.status(400).json({ error: errorMessages.EMAIL_REQUIRED });
  }

  try {
    const result = await userService.sendRecoveryEmail(email);

    logger.info(`Recovery email sent successfully. Result: ${result}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.status(200).json({ message: result });
  } catch (error: any) {
    logger.error("Ocorreu um erro na rota", { error: error });
    res.status(500).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};

export const loginUserController = async (req: Request, res: Response) => {
  try {
    const { email, password } = req.body;

    const user = await userService.loginUser(email, password);

    logger.info(`User logged in successfully. User: ${JSON.stringify(user)}`);
    logger.info(`Request Body: ${JSON.stringify(req.body)}`);

    res.json(user);
  } catch (error: any) {
    logger.error(
      `Ocorreu um erro na rota: ${error.message}, body: ${JSON.stringify(
        req.body
      )}`,
      {
        stack: error.stack,
      }
    );
    res.status(500).json({
      error: new AppError(error.message, error.statusCode),
    });
  }
};
