import {Event, EventStatus, UserType} from "@prisma/client";
import {endOfDay, parseISO, startOfDay} from "date-fns";
import {EventRegistrationData} from "../../interfaces/eventInterface";
import {prisma} from "../../utils/prisma";

type EventsQueryParams = {
  startTime?: string;
  authorId?: number;
  userType?: number;
  date?: string;
  patientId?: number;
};

export async function getTheLastFourEventsByUser(userId: number) {
  return prisma.event.findMany({
    include: {
      author: true,
    },
    take: 4,
    where: {
      patientId: userId,
      AND: [
        {
          active: true,
        },
        {
          status: {
            in: [
              EventStatus.WAITING_CONFIRMATION,
              EventStatus.CONFIRMED,
              EventStatus.FINISHED,
              EventStatus.MISSED,
            ],
          },
        },
      ],
    },
    orderBy: {
      date: `desc`,
    },
  });
}

export async function linkUserToAEvent(id: number, userId: number) {
  return prisma.event.update({
    where: {
      id: id,
    },
    data: {
      patientId: userId,
      status: EventStatus.WAITING_CONFIRMATION,
    },
  });
}

export const createManyEvents = async (eventsData: Event[]) => {
  return prisma.event.createMany({
    data: eventsData
  });
};


export const updateEvent = async (
  id: number,
  eventData: Partial<EventRegistrationData>
) => {
  const {
    patientId,
    eventType,
    name,
    date,
    startTime,
    endTime,
    status,
  } = eventData;

  const updateData: any = {
    updatedAt: new Date(),
  };

  if (patientId !== undefined) {
    updateData.patientId = patientId;
  }

  if (eventType !== undefined) {
    updateData.eventType = eventType;
  }

  if (name !== undefined) {
    updateData.name = name;
  }

  if (date !== undefined) {
    updateData.date = new Date(date);
  }

  if (startTime !== undefined) {
    updateData.startTime = new Date(startTime);
  }

  if (endTime !== undefined) {
    updateData.endTime = new Date(endTime);
  }

  if (status !== undefined) {
    updateData.status = status;
  }

  if (Object.keys(updateData).length === 1) {
    throw new Error("No fields to update.");
  }

  return prisma.event.update({
    where: {id},
    data: updateData,
  });
};

export const deleteEventById = async (id: number) => {
  return prisma.event.update({
    where: {id},
    data: {
      active: false,
    },
    select: {
      id: true,
      active: true,
    },
  });
};

export const getEventById = async (eventId: number) => {
  return prisma.event.findFirst({
    where: {
      id: eventId,
    },
  });
};

export const get = async (queryParams?: EventsQueryParams) => {
  const query: { [p: string]: string | UserType | number } = Object.entries(queryParams)
    .filter((t) => t[1])
    .reduce((acc, index) => {
      const [key, value] = index;

      return {
        ...acc,
        [key]: value,
      };
    }, {});

  let where: Omit<EventsQueryParams, "date"> & {
    author?: { userType: UserType };
    date?: { gte: string; lte: string };
  } = {};

  if (query.authorId) {
    where.authorId = +query.authorId;
  }

  if (query.startTime) {
    where.startTime = query.startTime.toString();
  }

  if (query.userType) {
    where.author = {
      userType: query.userType as UserType,
    };
  }

  if (query.date) {
    const dateObj = parseISO(<string>query.date);
    where.date = {
      gte: startOfDay(dateObj).toISOString(),
      lte: endOfDay(dateObj).toISOString(),
    };
  }

  if (query.patientId) {
    where.patientId = +query.patientId;
  }

  return prisma.event.findMany({
    include: {
      author: true,
      patient: true
    },
    where,
  });
};