import {UserRegistrationData} from "../../interfaces/userInterface";
import {Prisma, PrismaClient, UserType} from "@prisma/client";

const prisma = new PrismaClient();

export const getUserByEmail = async (email: string) => {
  return prisma.user.findFirst({
    where: {
      email,
    },
  });
};

export const getUserById = async (userId: number) => {
  return prisma.user.findFirst({
    where: {
      id: userId,
      userType: {
        in: [UserType.PSYCHOTHERAPIST, UserType.INTEGRATIVE_THERAPIST, UserType.USER],
      },
    },
  });
};

export const registerUser = async (userData: {
  gender: string | null;
  active: boolean;
  createdAt: Date | null;
  password: string;
  phone: string | null;
  childName: string | null;
  id: number;
  userType: UserType | null;
  region: string | null;
  losingPhase: string | null;
  email: string;
  username: string | null;
  updatedAt: Date | null
}) => {
  const user = await prisma.user.create({
    data: userData as Prisma.UserCreateInput,
  });
  return user.id;
};

export const editUser = async (user: Partial<UserRegistrationData>) => {
  const { id, phone, region, losingPhase, childName, gender } = user;

  return prisma.user.update({
    where: {id},
    data: {
      ...(phone && {phone}),
      ...(region && {region}),
      ...(typeof losingPhase !== "undefined" && user.userType === 1 && {losingPhase}),
      ...(childName && {childName}),
      ...(gender && {gender}),
      updatedAt: new Date(),
    },
    select: {
      id: true,
      email: true,
      phone: true,
      region: true,
      losingPhase: true,
      username: true,
      active: true,
      userType: true,
      createdAt: true,
      updatedAt: true,
      childName: true,
      gender: true,
    },
  });
};

export const removeUser = async (userId: number) => {
  const dataAtual = new Date();

  return prisma.user.update({
    where: {
      id: userId,
    },
    data: {
      updatedAt: dataAtual,
      active: false,
    },
    select: {
      id: true,
    },
  });
};

export const listAllUsers = async (queryParams: any) => {
  const userWhereInput: Prisma.UserWhereInput = {
    active: true,
  };

  if (queryParams.region) {
    userWhereInput.region = queryParams.region;
  }

  if (queryParams.losingPhase) {
    userWhereInput.losingPhase = queryParams.losingPhase;
  }

  if (queryParams.userType) {
    userWhereInput.userType = queryParams.userType;
  }

  if (queryParams.createdAt) {
    userWhereInput.createdAt = {
      gte: new Date(queryParams.createdAt),
    };
  }

  if (queryParams.updatedAt) {
    userWhereInput.updatedAt = {
      gte: new Date(queryParams.updatedAt),
    };
  }

  if (queryParams.gender) {
    userWhereInput.gender = queryParams.gender;
  }

  return prisma.user.findMany({
    where: userWhereInput,
    orderBy: {
      id: 'asc',
    },
    select: {
      id: true,
      email: true,
      phone: true,
      region: true,
      losingPhase: true,
      username: true,
      userType: true,
      createdAt: true,
      updatedAt: true,
      childName: true,
      gender: true,
    },
  });
};

export const listAllProfessional = async () => {
  return prisma.user.findMany({
    where: {
      active: true,
      userType: {
        in: [UserType.PSYCHOTHERAPIST, UserType.INTEGRATIVE_THERAPIST],
      },
    },
    orderBy: {
      id: 'asc',
    },
    select: {
      id: true,
      phone: true,
      region: true,
      username: true,
      userType: true,
      gender: true,
      createdAt: true,
      updatedAt: true,
    },
  });
};

export const updateUserPassword = async (id: number, newPassword: string) => {
  return prisma.user.update({
    where: {
      id,
    },
    data: {
      password: newPassword,
    },
  });
}

export const listProfessionalByType = async (userType: UserType) => {
  return prisma.user.findMany({
    where: {
      userType: userType,
      active: true,
    },
    select: {
      id: true,
      username: true,
      email: true,
      phone: true,
      region: true,
      userType: true,
      gender: true,
      createdAt: true,
      updatedAt: true,
    },
  });
};

export const getUserName = async (email: string) => {
  try {
    const {username} = await prisma.user.findFirst({
      where: {
        email,
      },
    });
    return username;
  } catch (error) {
    console.error("Erro ao executar a consulta:", error);
    throw error;
  }
};
