import * as eventRepository from "../../repository/event/eventRepository";
import {getTheLastFourEventsByUser} from "../../repository/event/eventRepository";
import {EventRegistrationData} from "../../interfaces/eventInterface";
import {errorMessages} from "../../../messages/errorMessages";
import getUserFromToken from "../token/tokenService";
import * as userRepository from "../../repository/user/userRepository";
import {EventStatus, EventType, UserType} from "@prisma/client";
import {setHours} from "date-fns";

const toEventsDTO = (data: any[]) =>
  data.reduce<EventsResponse[]>((acc, item) => {
    const [date] = item.date.toISOString().split("T");

    const listIndex = acc.findIndex((event) => date === event.date);

    const patient = item.patient ? {id: item.patient.id} : null;

    const event = {
      description: item.name,
      endTime: item.endTime,
      id: item.id,
      professional: {
        name: item.author.username,
        speciality: String(item.author.userType),
        id: item.author.id,
      },
      patient,
      startTime: item.startTime,
      status: item.status,
      title: item.name,
    };

    if (listIndex < 0) {
      acc.push({
        date,
        events: [event],
      });
      return acc;
    }

    acc[listIndex].events.push(event);
    return acc;
  }, []);

export type EventResponse = {
  id: number;
  status: string;
  title: string;
  description: string;
  startTime: string;
  endTime: string;
  professional: {
    name: string;
    speciality: string;
    id: number;
  };
  patient?: {
    id: number;
  };
};

type EventsResponse = {
  date: string;
  events: EventResponse[];
};

export async function linkUserToAEvent(id: string, token: string) {
  const userId = getUserFromToken(token);
  if (!id || !userId) {
    throw new Error(errorMessages.ID_NOT_FOUND);
  }

  const event = await eventRepository.getEventById(+id);
  if (!event) {
    throw new Error(errorMessages.EVENT_NOT_FOUND);
  }

  const lastFourEvents = await getTheLastFourEventsByUser(userId);

  if (lastFourEvents.length) {
    const idLastAuthor = lastFourEvents[0]?.authorId;
    const filteredEvents = lastFourEvents.filter(
      (el) => el.authorId === idLastAuthor
    );

    if (filteredEvents.length < 4) {
      if (idLastAuthor !== event.authorId) {
        throw new Error("O usuário já tem vínculo com outro profissional.");
      }
    }
  }

  return await eventRepository.linkUserToAEvent(+id, userId);
}

export const createManyEvents = async (
  eventData: { date: string; hours: string[] },
  token: string
) => {

  function validateHour(hours: string[]) {
    const hoursSet = new Set(hours);
    hours.forEach((hour) => {
      if (!hour.match(/^(0[0-9]|1[0-9]|2[0-3]):00$/)) {
        throw new Error("Não é possível criar eventos com horários inválidos.");
      }
    });
    if (hoursSet.size !== hours.length) {
      throw new Error("Não é possível criar eventos com horários duplicados.");
    }
  }

  validateHour(eventData.hours);

  const userId = getUserFromToken(token);

  const user = await userRepository.getUserById(userId);
  if (!user) {
    throw new Error(
      "(nAuthorID) Somente Psicólogo/ Terapeuta podem criar agendamentos."
    );
  }
  if (!!eventData.date) {
    const dataAtual = new Date();
    const dataEvento = new Date(eventData.date);
    if (dataEvento < dataAtual) {
      throw new Error("Não é possível alterar agendas para datas passadas.");
    }
    eventData.date = dataEvento.toISOString();
  }

  return eventRepository.createManyEvents(
    eventData.hours.map((hour) => {
      return {
        authorId: userId,
        date: setHours(new Date(eventData.date), 12),
        startTime: hour,
        endTime: String(+hour + 1),
        active: true,
        eventType: user.userType === UserType.PSYCHOTHERAPIST ? EventType.PSYCHOTHERAPY : EventType.INTEGRATIVE_THERAPY,
        name: undefined,
        id: undefined,
        status: EventStatus.AVAILABLE,
        patientId: undefined,
        updatedAt: undefined,
      };
    })
  );
};

export const updateEvent = async (
  id: number,
  eventData: Partial<EventRegistrationData>
) => {
  const existingEvent = await eventRepository.getEventById(id);
  if (!existingEvent) {
    throw new Error(errorMessages.EVENT_NOT_FOUND);
  }
  return await eventRepository.updateEvent(id, eventData);
};

export const deleteEvent = async (id: number) => {
  const existingEvent = await eventRepository.getEventById(id);
  if (!existingEvent) {
    throw new Error(errorMessages.EVENT_NOT_FOUND);
  }

  return await eventRepository.deleteEventById(id);
};

type EventsQueryParams = {
  startTime?: string;
  authorId?: number;
  userType?: number;
  date?: string;
  patientId?: number;
};

export const get = async (queryParams: EventsQueryParams) => {
  const result = await eventRepository.get(queryParams);
  return toEventsDTO(result);
};
