import crypto from "crypto";
import * as userRepository from "../../repository/user/userRepository";
import { UserRegistrationData } from "../../interfaces/userInterface";
import { registerUser } from "./userService";
import { errorMessages } from "../../../messages/errorMessages";

jest.mock("../../repository/user/userRepository");

const mockGetUserByEmail = userRepository.getUserByEmail as jest.MockedFunction<
  typeof userRepository.getUserByEmail
>;

const mockRegisterUser = userRepository.registerUser as jest.MockedFunction<
  typeof userRepository.registerUser
>;

const generateUniqueEmail = () => {
  const randomValue = Math.random().toString(36).substring(2, 15);
  return `test-${randomValue}@example.com`;
};
const sampleUserData: UserRegistrationData = {
    strEmail: "test@example.com",
    strUsername: "testuser",
    strPassword: "testpassword",
    confirmPassword: "testpassword",
    nUserType: 1,
    strPhone: "",
    strRegion: "",
    strLosingPhase: "",
    nUserID: 1,
    strChildrenName: "testchildrenname",
  };

  describe("userService", () => {
    beforeEach(() => {
      mockGetUserByEmail.mockClear();
      mockRegisterUser.mockClear();
    });

  it("registerUser should register a new user and return the user ID", async () => {
    const uniqueEmail = generateUniqueEmail();
    const userData: UserRegistrationData = {
      ...sampleUserData,
      strEmail: uniqueEmail,
    };

    mockGetUserByEmail.mockResolvedValue(null);

    const mockDigest = jest.fn();
    mockDigest.mockReturnValue("hashedPassword");

    jest.spyOn(crypto, "createHash").mockReturnValue({
      update: jest.fn().mockReturnThis(),
      digest: mockDigest,
    } as unknown as crypto.Hash);

    mockRegisterUser.mockResolvedValue(1);

    const userId = await registerUser(userData);

    expect(mockGetUserByEmail).toHaveBeenCalledWith(uniqueEmail);
    expect(crypto.createHash).toHaveBeenCalledWith("sha256");
    expect(mockDigest).toHaveBeenCalledWith("hex");
    expect(mockRegisterUser).toHaveBeenCalledWith({
      ...userData,
      strPassword: "hashedPassword",
    });
    expect(userId).toEqual(1);
  });

  it("registerUser should throw an error if the email is already in use", async () => {
    const uniqueEmail = generateUniqueEmail();
    const userData: UserRegistrationData = {
      ...sampleUserData,
      strEmail: uniqueEmail,
    };

    mockGetUserByEmail.mockResolvedValue({});

    await expect(registerUser(userData)).rejects.toThrow(
      errorMessages.EMAIL_ALREADY_USED
    );

    expect(mockGetUserByEmail).toHaveBeenCalledWith(uniqueEmail);
    expect(mockRegisterUser).not.toHaveBeenCalled();
  });

  it("registerUser should throw an error if password and confirmPassword do not match", async () => {
    const uniqueEmail = generateUniqueEmail();
    const userData: UserRegistrationData = {
      ...sampleUserData,
      strEmail: uniqueEmail,
      confirmPassword: "mismatchedPassword",
    };

    await expect(registerUser(userData)).rejects.toThrow(
      errorMessages.PASSWORD_MISMATCH
    );

    expect(mockGetUserByEmail).not.toHaveBeenCalled();
    expect(mockRegisterUser).not.toHaveBeenCalled();
  });

  it("registerUser should throw an error if any required field is missing", async () => {
    const uniqueEmail = generateUniqueEmail();
    const userData: UserRegistrationData = {
      ...sampleUserData,
      strUsername: "",
    };

    await expect(registerUser(userData)).rejects.toThrow(
      errorMessages.MISSING_FIELDS
    );

    expect(mockGetUserByEmail).not.toHaveBeenCalled();
    expect(mockRegisterUser).not.toHaveBeenCalled();
  });

  it("registerUser should throw an error if the email format is invalid", async () => {
    const uniqueEmail = generateUniqueEmail();
    const userData: UserRegistrationData = {
      ...sampleUserData,
      strEmail: "invalid-email",
    };

    await expect(registerUser(userData)).rejects.toThrow(
      errorMessages.INVALID_EMAIL_FORMAT
    );

    expect(mockGetUserByEmail).not.toHaveBeenCalled();
    expect(mockRegisterUser).not.toHaveBeenCalled();
  });
});
