/* eslint-disable @typescript-eslint/no-var-requires */
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/no-explicit-any */
import crypto from "crypto";
import * as userRepository from "../../repository/user/userRepository";
import { UserRegistrationData } from "../../interfaces/userInterface";
import * as randomString from "../../utils/randomStringGenerator";
import { errorMessages } from "../../../messages/errorMessages";
import { successMessages } from "../../../messages/successMessages";
import nodemailer from "nodemailer";
import {
  generateAccessToken,
  generateRefreshToken,
} from "../../utils/generateToken";
import {User, UserType} from "@prisma/client";

export const registerUser = async (
  userData: User & { confirmPassword: string }
) => {
  const { confirmPassword, ...user } = userData;
  const emailRegex = /^[^\s@]+@[^\s@]+\.[^\s@]+$/;

  if (!user.email || !user.username || !user.password || !user.userType) {
    throw new Error(errorMessages.MISSING_FIELDS);
  }

  if (!user.email.match(emailRegex)) {
    throw new Error(errorMessages.INVALID_EMAIL_FORMAT);
  }

  if (user.password !== confirmPassword) {
    throw new Error(errorMessages.PASSWORD_MISMATCH);
  }

  const existingUser = await userRepository.getUserByEmail(user.email);
  if (existingUser) {
    throw new Error(errorMessages.EMAIL_ALREADY_USED);
  }

  const hashedPassword = crypto
    .createHash("sha256")
    .update(user.password)
    .digest("hex");

  return await userRepository.registerUser({
    ...user,
    password: hashedPassword,
    active: true,
  });
};

export const loginUser = async (email: string, password: string) => {
  if (!email || !password) {
    throw new Error(errorMessages.MISSING_FIELDS);
  }

  const hashedPassword = crypto
    .createHash("sha256")
    .update(password)
    .digest("hex");

  try {
    const user = await userRepository.getUserByEmail(email);
    if (!user) {
      throw new Error(errorMessages.USER_NOT_FOUND);
    }

    if (!user.active) {
      throw new Error(errorMessages.INACTIVE_USER);
    }

    if (hashedPassword === user.password) {
      const refreshToken = generateRefreshToken({
        id: user.id,
        userType: user.userType,
      });
      const accessToken = generateAccessToken(user);
      return {
        accessToken,
        refreshToken,
        user: {
          name: user.username,
          email: user.email,
          id: user.id,
        },
      };
    }
  } catch (error: any) {
    if (error.message === "Invalid password") {
      throw new Error(errorMessages.INVALID_PASSWORD);
    } else {
      throw error;
    }
  }
};

export const removeUser = async (userId: number) => {
  if (!userId) {
    throw new Error(errorMessages.ID_NOT_FOUND);
  }

  const existingUser = await userRepository.getUserById(userId);

  if (!existingUser) {
    throw new Error(errorMessages.USER_NOT_FOUND);
  }

  return await userRepository.removeUser(userId);
};

export const listAllUsers = async (queryParams: any) => {
  const result = await userRepository.listAllUsers(queryParams);
  if (result.length === 0) {
    throw new Error(errorMessages.USERS_NOT_FOUND);
  }

  return result;
};

export const listAllProfessional = async () => {
  const result = await userRepository.listAllProfessional();
  if (result.length == 0) {
    throw new Error(errorMessages.USERS_NOT_FOUND);
  }
  return result;
};

export const listProfessionalByType = async (userType: UserType) => {

  if (!userType) {
    throw new Error(errorMessages.ID_NOT_FOUND)
  }

  const result = await userRepository.listProfessionalByType(userType);
  if (result == null) {
    throw new Error(errorMessages.PROFESSIONAL_NOT_FOUND);
  }
  return result;
}

export const editUser = async (userData: UserRegistrationData) => {
  if (!userData) {
    throw new Error(errorMessages.MISSING_FIELDS);
  }

  const existingUser = await userRepository.getUserById(userData.id);
  if (!existingUser) {
    throw new Error(errorMessages.USER_NOT_FOUND);
  }

  return await userRepository.editUser(userData);
};

export const listUserByID = async (nUserId: number) => {
  if (!nUserId) {
    throw new Error(errorMessages.ID_NOT_FOUND);
  }

  const result = await userRepository.getUserById(nUserId);
  if (!result) {
    throw new Error(errorMessages.USER_NOT_FOUND);
  }

  return result;
};

export const sendRecoveryEmail = async (to: string) => {
  try {
    require("dotenv").config();
    const newPass = randomString.generateRandomString(16);

    const hashedPassword = crypto
      .createHash("sha256")
      .update(newPass)
      .digest("hex");

    const existingUser = await userRepository.getUserByEmail(to);
    if (!existingUser) {
      throw new Error(errorMessages.USER_NOT_FOUND);
    }
    await userRepository.updateUserPassword(existingUser.id, hashedPassword);

    const transport = nodemailer.createTransport({
      service: process.env.SMTP_SERVICE,
      auth: {
        user: process.env.SMTP_USER,
        pass: process.env.SMTP_PASSWORD,
      },
    });

    const userName = await userRepository.getUserName(to);
    const mailOptions = {
      from: process.env.SMTP_USER,
      to: to,
      subject: "Recuperação de senha",
      text: `${userName}, sua nova senha: ${newPass}`,
    };

    await transport.sendMail(mailOptions, (error: any, info: any) => {
      if (error) {
        throw new Error(errorMessages.EMAIL_SEND_ERROR);
      }
    });

    return successMessages.EMAIL_SENT_SUCCESS;
  } catch (error) {
    throw new Error(errorMessages.REQUEST_PROCESSING_ERROR);
  }
};
function passwordMatch(password: string, strpassword: any) {
  throw new Error("Function not implemented.");
}
